-----------------------------------------------------------------------------------------------
-- Client Lua Script for Wildminer
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------

require "Window"
require "Unit"

-----------------------------------------------------------------------------------------------
-- Wildminer Module Definition
-----------------------------------------------------------------------------------------------
local Wildminer = {}
local kVersion = 4

local kYesToLocale = {
	Yes = "en",
	Ja = "de",
	Oui = "fr"
}

local kMaxUnits = 20000

-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function Wildminer:new(o)
	o = o or {}
	setmetatable(o, self)
	self.__index = self

	-- initialize variables here
	self.tLastVendorUnit = nil
	self.nUnitsCount = 0
	self.tData = {
		units = {},
		challenges = {}
	}
	
	self.tUIStats = {
		kills = 0,
		positions = 0,
		drops = 0
	}

	return o
end

function Wildminer:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
	Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end


-----------------------------------------------------------------------------------------------
-- Wildminer OnLoad
-----------------------------------------------------------------------------------------------
function Wildminer:OnLoad()
	-- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("Wildminer.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- Wildminer OnDocLoaded
-----------------------------------------------------------------------------------------------
function Wildminer:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
		self.wndMain = Apollo.LoadForm(self.xmlDoc, "WildminerForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end

		self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil

		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterEventHandler("UnitCreated", "OnUnitCreated", self)
		Apollo.RegisterEventHandler("CombatLogDamage", "OnCombatLogDamage", self)
		Apollo.RegisterEventHandler("VendorItemsUpdated","OnVendorItemsUpdated", self)
		Apollo.RegisterEventHandler("InvokeVendorWindow","OnInvokeVendorWindow", self)

		Apollo.RegisterEventHandler("ChallengeRewardListReady", "OnChallengeRewardListReady", self)
		
		Apollo.RegisterSlashCommand("wildminer", "OnShowWildminer", self)

		-- Do additional Addon initialization here
	end
end

-----------------------------------------------------------------------------------------------
-- Wildminer Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-----------------------------------------------------------------------------------------------
-- Wildminer CreateUnit
-----------------------------------------------------------------------------------------------
function Wildminer:OnShowWildminer(unit)
	self.wndMain:Show(true)
end

-----------------------------------------------------------------------------------------------
-- Wildminer UpdateUI
-----------------------------------------------------------------------------------------------
function Wildminer:UpdateUI()
	self.wndMain:FindChild("KillCount"):SetText(tostring(self.tUIStats.kills))
	self.wndMain:FindChild("DropCount"):SetText(tostring(self.tUIStats.drops))
	self.wndMain:FindChild("PositionCount"):SetText(tostring(self.tUIStats.positions))
end

-----------------------------------------------------------------------------------------------
-- Wildminer CreateUnit
-----------------------------------------------------------------------------------------------
function Wildminer:CreateUnit(unit)
	if kMaxUnits < self.nUnitsCount then
		return nil
	end
	self.nUnitsCount = self.nUnitsCount + 1
	local tUnit = {
		name = unit:GetName(),
		type = unit:GetType()
	}
	if tUnit.type == "Player" or tUnit.type == "PinataLoot" then
		return nil
	end
	local tZone = GameLib.GetCurrentZoneMap()
	if tZone ~= nil then
		tUnit.zid = tZone.id
	end
	tUnit.vet = false
	local tInstanceSettings = GameLib.GetInstanceSettings()
	if tInstanceSettings ~= nil and tInstanceSettings.eWorldDifficulty == GroupLib.Difficulty.Veteran then
		tUnit.vet = true
	end
	local tPos = unit:GetPosition()
	if tPos ~= nil then
		tUnit.pos = tPos.x .. "/" .. tPos.z
	end
	tUnit.lvl = unit:GetLevel()
	if tUnit.type == "NonPlayer" then
		tUnit.aff = unit:GetAffiliationName()
		tUnit.elite = unit:GetEliteness()
		tUnit.faction = unit:GetFaction()
		tUnit.health = unit:GetMaxHealth()
		tUnit.shield = unit:GetShieldCapacityMax()
		tUnit.abs = unit:GetAbsorptionMax()
		tUnit.race = unit:GetUnitRaceId()
		tUnit.rank = unit:GetRank()
		tUnit.gender = unit:GetGender()
		local tArchetype = unit:GetArchetype()
		if tArchetype ~= nil then
			tUnit.arch = tArchetype.idArchetype
		end
	elseif tUnit.type == "Harvest" then
		tUnit.trdsknm = unit:GetHarvestRequiredTradeskillName()
		tUnit.trdsktr = unit:GetHarvestRequiredTradeskillTier()
	end
	return tUnit
end

-----------------------------------------------------------------------------------------------
-- Wildminer GetUnit
-----------------------------------------------------------------------------------------------
function Wildminer:GetUnit(unit)
	if unit ~= nil and unit:IsValid() then
		local nId = unit:GetId()
		local sName = unit:GetName()
		local sType = unit:GetType()
		if nId ~= nil and sName ~= nil and sType ~= nil then
			local tUnits = self.tData.units[nId]
			if tUnits ~= nil then
				local tCurrentUnit = tUnits[table.maxn(tUnits)]
				if tCurrentUnit.name == sName and tCurrentUnit.type == sType then
					return tCurrentUnit
				end
			end
			local tUnit = self:CreateUnit(unit)
			if tUnit ~= nil then
				if tUnits == nil then
					tUnits = {}
					self.tData.units[nId] = tUnits
				end
				table.insert(tUnits, tUnit)
				return tUnit
			end
		end
	end
	return nil
end

-----------------------------------------------------------------------------------------------
-- Wildminer GetUnitById
-----------------------------------------------------------------------------------------------
function Wildminer:GetUnitById(nId)
	local tUnits = self.tData.units[nId]
	if tUnits ~= nil then
		return tUnits[table.maxn(tUnits)]
	end
	return nil
end

-----------------------------------------------------------------------------------------------
-- Wildminer RegisterUnitDrop
-----------------------------------------------------------------------------------------------
function Wildminer:RegisterUnitDrop(unit)
	local tLoot = unit:GetLoot()
	-- If it's a loot
	if tLoot ~= nil then
		local eType = tLoot.eLootItemType
		local nOwnerId = tLoot.idOwner
		if eType ~= nil and nOwnerId ~= nil then
			local tOwnerUnit = self:GetUnitById(nOwnerId)
			if tOwnerUnit ~= nil then
				if tOwnerUnit.drops == nil then
					tOwnerUnit.drops = {
						items = {},
						vitems = {},
						spells = {},
						cur = {}
					}
				end
				local tDrops = tOwnerUnit.drops
				if eType == Unit.CodeEnumLootItemType.StaticItem then
					if tLoot.itemLoot ~= nil then
						local nItemId = tLoot.itemLoot:GetItemId()
						if nItemId ~= nil then
							if tDrops.items[nItemId] == nil then
								tDrops.items[nItemId] = 0
							end
							local nCount = tLoot.nCount
							if nCount == nil then
								nCount = 1
							end
							tDrops.items[nItemId] = tDrops.items[nItemId] + nCount
						end
					end
				elseif eType == Unit.CodeEnumLootItemType.VirtualItem then
					local sName = tLoot.strName
					if sName ~= nil then
						local sVItemHash = table.concat({
							sName,
							tLoot.eType or 0
						}, "$")
						if tDrops.vitems[sVItemHash] == nil then
							tDrops.vitems[sVItemHash] = 0
						end
						local nCount = tLoot.nCount
						if nCount == nil then
							nCount = 1
						end
						tDrops.vitems[sVItemHash] = tDrops.vitems[sVItemHash] + nCount
					end
				elseif eType == Unit.CodeEnumLootItemType.Cash then
					if tLoot.monCurrency ~= nil then
						local nMoneyType = tLoot.monCurrency:GetMoneyType()
						if nMoneyType ~= nil then
							local sCurrencyHash = tostring(nMoneyType)
							if tDrops.cur[sCurrencyHash] == nil then
								tDrops.cur[sCurrencyHash] = 0
							end
							local nAmount = tLoot.monCurrency:GetAmount()
							if nAmount == nil then
								nAmount = 1
							end
							tDrops.cur[sCurrencyHash] = tDrops.cur[sCurrencyHash] + nAmount
						end
					end
				elseif eType == Unit.CodeEnumLootItemType.AdventureSpell then
					if tLoot.tAbility ~= nil then
						local sName = tLoot.tAbility.strName
						if sName ~= nil then
							if tDrops.spells[sName] == nil then
								tDrops.spells[sName] = 0
							end
							tDrops.spells[sName] = tDrops.spells[sName] + 1
						end
					end
				end
			end
		end
	end
end

-----------------------------------------------------------------------------------------------
-- Wildminer OnUnitCreated
-----------------------------------------------------------------------------------------------
function Wildminer:OnUnitCreated(unit)
	if unit ~= nil and unit:IsValid() then
		local sType = unit:GetType()
		if sType == "PinataLoot" then
			self:RegisterUnitDrop(unit)
			self.tUIStats.drops = self.tUIStats.drops + 1
		else
			-- Dummy get to create the units
			local tUnit = self:GetUnit(unit)
			self.tUIStats.positions = self.tUIStats.positions + 1
		end
		self:UpdateUI()
	end
end

-----------------------------------------------------------------------------------------------
-- Wildminer OnCombatLogDamage
-----------------------------------------------------------------------------------------------
function Wildminer:OnCombatLogDamage(tEventArgs)
	if tEventArgs.bTargetKilled and tEventArgs.unitTarget ~= nil then
		local tUnit = self:GetUnit(tEventArgs.unitTarget)
		if tUnit ~= nil then
			if tUnit.kc == nil then
				tUnit.kc = 0
			end
			tUnit.kc = tUnit.kc + 1
			self.tUIStats.kills = self.tUIStats.kills + 1
			self:UpdateUI()
		end
	end

	if tEventArgs.unitCaster ~= nil and tEventArgs.splCallingSpell ~= nil then
		local tUnitCaster = self:GetUnit(tEventArgs.unitCaster)
		if tUnitCaster ~= nil then
			local nSpellId = tEventArgs.splCallingSpell:GetId()
			if nSpellId ~= nil then
				if tUnitCaster.casts == nil then
					tUnitCaster.casts = {}
				end
				tUnitCaster.casts[nSpellId] = true
			end
		end
	end
end

-----------------------------------------------------------------------------------------------
-- Wildminer OnVendorItemsUpdated
-----------------------------------------------------------------------------------------------
function Wildminer:OnVendorItemsUpdated()
	if self.tLastVendorUnit ~= nil then
		local tItems = self.tLastVendorUnit:GetVendorItems()
		if tItems then
			local tLastVendorUnit = self:GetUnit(self.tLastVendorUnit)
			if tLastVendorUnit ~= nil then
				if tLastVendorUnit.sells == nil then
					tLastVendorUnit.sells = {
						items = {},
						spells = {}
					}
				end
				for key, tCurrItem in pairs(tItems) do
					local tPriceInfo = tCurrItem.tPriceInfo
					if tPriceInfo ~= nil then
						local tPrice = {
							stack = tCurrItem.nStackSize or 1
						}
						if tPriceInfo.nAmount1 ~= nil  and tPriceInfo.nAmount1 ~= 0 then
							tPrice.cur1 = tPriceInfo.eCurrencyType1
							tPrice.am1 = tPrice.stack * tPriceInfo.nAmount1
							tPrice.alt1 = tPriceInfo.eAltType1
						end
						if tPriceInfo.nAmount2 ~= nil  and tPriceInfo.nAmount2 ~= 0 then
							tPrice.cur2 = tPriceInfo.eCurrencyType2
							tPrice.am2 = tPrice.stack * tPriceInfo.nAmount2
							tPrice.alt2 = tPriceInfo.eAltType2
						end
						if tCurrItem.itemData ~= nil and tCurrItem.itemData:GetItemId() ~= nil then
							tLastVendorUnit.sells.items[tCurrItem.itemData:GetItemId()] = tPrice
						end
						if tCurrItem.splData ~= nil and tCurrItem.splData:GetId() ~= nil then
							tLastVendorUnit.sells.spells[tCurrItem.splData:GetId()] = tPrice
						end
					end
				end
			end
		end
	end
end

-----------------------------------------------------------------------------------------------
-- Wildminer OnInvokeVendorWindow
-----------------------------------------------------------------------------------------------
function Wildminer:OnInvokeVendorWindow(unit)
	if unit ~= nil and unit:IsValid() then
		self.tLastVendorUnit = unit
	end
end

-----------------------------------------------------------------------------------------------
-- Wildminer OnChallengeRewardListReady
-----------------------------------------------------------------------------------------------
function Wildminer:OnChallengeRewardListReady(idChallenge, iRewardTier)
	local tRewardList = ChallengesLib.GetRewardList(idChallenge)
	if tRewardList ~= nil then
		for idx, tReward in ipairs(tRewardList) do
			if tReward ~= nil and tReward.itemReward ~= nil and tReward.itemReward:GetItemId() ~= nil and tReward.nChallengeTier ~= nil and tReward.nAmount ~= nil then
				local tChallenges = self.tData.challenges
				if tChallenges[idChallenge] == nil then
					tChallenges[idChallenge] = {}
				end
				local tChallenge = tChallenges[idChallenge]
				if tChallenge[tReward.nChallengeTier] == nil then
					tChallenge[tReward.nChallengeTier] = {}
				end
				tChallenge[tReward.nChallengeTier][tReward.itemReward:GetItemId()] = tReward.nAmount
			end
		end
	end
end

-----------------------------------------------------------------------------------------------
-- Wildminer OnSave
-----------------------------------------------------------------------------------------------
function Wildminer:OnSave(level)
	if level == GameLib.CodeEnumAddonSaveLevel.General then
		local sLocale = "unk"
		local sYesResult = Apollo.GetString("Yes")
		if sYesResult ~= nil and kYesToLocale[sYesResult] ~= nil then
			sLocale = kYesToLocale[sYesResult]
		end
		return {
			version = kVersion,
			locale = sLocale,
			data = self.tData
		}
	end
end

-- No restore, if the app was not run until the second save, just overwrite

-----------------------------------------------------------------------------------------------
-- WildminerForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function Wildminer:OnOK()
	self.wndMain:Close() -- hide the window
end

-- when the Cancel button is clicked
function Wildminer:OnCancel()
	self.wndMain:Close() -- hide the window
end


-----------------------------------------------------------------------------------------------
-- Wildminer Instance
-----------------------------------------------------------------------------------------------
local WildminerInst = Wildminer:new()
WildminerInst:Init()
